use std::{
    error::Error,
    fs::File,
    io::{prelude::*, BufReader},
};

/// Enum used to specify input
#[allow(unused)]
pub enum InputType {
    Test1,
    Test2,
    Test3,
}

/// Function to get the input for a given leetcode problem
pub fn get_input(id: u32, input: InputType) -> Result<Vec<String>, Box<dyn Error>> {
    // Find Input File Name
    let file_name = match input {
        InputType::Test1 => format!("src/input/{}_test1", id),
        InputType::Test2 => format!("src/input/{}_test2", id),
        InputType::Test3 => format!("src/input/{}_test3", id),
    };

    // Read file into buffer
    let file = File::open(file_name)?;
    let reader = BufReader::new(file);

    // Convert itterate buffer and return string vec
    let data = reader
        .lines()
        .map(|s| s.unwrap_or(String::from("")))
        .collect();

    Ok(data)
}

//
// Below this line are a series of utils that can be re-used
// for any of the problems identified
#[allow(unused)]
//
///=========================================================
//

pub fn lcm(first: u64, second: u64) -> u64 {
    first * second / gcd(first, second)
}

pub fn gcd(first: u64, second: u64) -> u64 {
    let mut max = first;
    let mut min = second;
    if min > max {
        std::mem::swap(&mut max, &mut min);
    }

    loop {
        let res = max % min;
        if res == 0 {
            return min;
        }

        max = min;
        min = res;
    }
}
